from gpkgstatus.JSONFileReader import JSONFileReader, FileNotFoundException
from gpkgstatus.URLReader import URLReader
from termcolor import colored


def select_url(name: str, version: int = None):
    first_letter = version[0]
    urls = {
        "f": f"https://bodhi.fedoraproject.org/updates/?search={name}",
    }

    if first_letter in urls:
        url = urls[first_letter]
    else:
        print(colored("Error: Invalid Distribution. Format: f{version}", "red"))
        exit(1)

    if len(version) > 1:
        url += f"&releases={version}"
    return url


def print_update_info(update: dict, status_color: str):
    colors = {"stable": "green", "testing": "yellow", "pending": "red"}

    if update["status"].lower() in colors:
        status_color = colors[update["status"]]
    else:
        status_color = "blue"

    print(colored(f"Update ID: {update['updateid']}", status_color))
    print(colored(f"Package Name: {update['title']}", status_color))
    print(colored(f"Status: {update['status']}", status_color))
    print("------------------------------")


def main(args: dict):
    cache_time = 3600  # 1 hr
    distro_version = args["distro_version"][0]

    try:
        limit = int(args["limit"][0])
    except ValueError:
        print(colored("You must enter an integer value.", "red"))
        exit(0)

    cache_file = f"{args['name']}_{distro_version}.json"
    url = select_url(args["name"], distro_version.lower())

    try:
        file_reader = JSONFileReader(cache_file, "updates")

        if args["force"] or (file_reader.relative_time() > cache_time):
            url_reader = URLReader(url)
            url_reader.save_as_file(cache_file)

    except FileNotFoundException:
        url_reader = URLReader(url)
        url_reader.save_as_file(cache_file)
        file_reader = JSONFileReader(cache_file, "updates")

    finally:
        updates = file_reader.read(limit)

        if not updates:
            print(colored("No Updates Found. Check your arguments.", "red"))
            exit(0)

        for update in updates:
            print_update_info(update, "green")



from gpkgstatus.JSONFileReader import JSONFileReader, FileNotFoundException
from pytest import raises


class TestJSONFileReader:
    def test_less_keys(self):
        with raises(SystemExit):
            JSONFileReader("test.py")

    def test_invalid_file(self):
        with raises(FileNotFoundException):
            keys = ["test", "dev", "beta"]
            JSONFileReader("test.py", *keys)



import os

from json import load as jsonload
from pathlib import Path
from sys import exit
from tempfile import gettempdir
from termcolor import colored
from time import time
from typing import Union


class FileNotFoundException(Exception):
    pass


class JSONFileReader:
    _keys: list = []
    _path: Path = Path()

    def __init__(self, _path: Union[Path, str], *_keys):
        if len(_keys) < 1:
            print(colored("Error: At least one key is required"))
            exit(1)

        if not isinstance(_path, Path):
            _path = Path(os.path.join(gettempdir(), _path))

        if not (_path.exists() and _path.is_file()):
            raise FileNotFoundException()

        self._keys = list(_keys)
        self._path = _path

    def read(self, limit: int = 5):
        try:
            with open(self._path, "r") as file:
                data = jsonload(file)
                searched_info = []

                for key in self._keys:
                    if key not in data.keys():
                        print(colored(f"Warning: Key {key} doesn't exist"))
                    else:
                        for value in data[key]:
                            if len(searched_info) >= limit:
                                return searched_info

                            searched_info.append(value)
        except PermissionError:
            print(
                colored(
                    "Error: Permission denied. Please check temp directory permissions.",
                    "red",
                )
            )
            exit(1)

        return searched_info

    def relative_time(self):
        return time() - os.path.getmtime(self._path)



import json
import requests

from os.path import join
from sys import exit
from tempfile import gettempdir
from termcolor import colored


class URLReader:
    class _StatusException(Exception):
        pass

    _url: str = ""

    def __init__(self, _url: str):
        try:
            response = requests.head(_url)

            if response.status_code != 200:
                print(
                    colored("Error: Website isn't returning HTTP Status Code (200 OK)")
                )
                raise URLReader._StatusException()

            self._url = _url

        except requests.ConnectionError:
            print(colored(f"Error: Could not connect to {_url}", "red"))
            exit(1)

    def __load_json(self):
        try:
            response = requests.get(self._url)
            return response.json()
        except:
            print(
                colored(
                    "Error: There is an issue parsing JSON from corresponding website. Check if given URL is website JSON URL",
                    "red",
                )
            )
            exit(1)

    def save_as_file(self, filename: str, data: str = None):
        if data is None:
            data = self.__load_json()

        temp_file = join(gettempdir(), filename)

        try:
            with open(temp_file, "w") as file:
                json.dump(data, file, skipkeys=True, indent=4)
        except PermissionError:
            print(
                colored(
                    "Error: Permission denied. Please check temp directory permissions.",
                    "red",
                )
            )
            exit(1)



from pytest import raises

import gpkgstatus.gpkgstatus as gpkg


class TestFedoraUpdates:
    def test_search_package(self, capsys):
        search_term = "kernel"

        gpkg.main(
            {"name": search_term, "distro_version": "f", "force": False, "limit": "5"}
        )

        captured = capsys.readouterr()
        words = (search_term, "Update ID", "Package Name", "Status")
        assert all(word in captured.out for word in words)

    def test_valid_distro(self, capsys):
        search_term = "python"

        gpkg.main(
            {"name": search_term, "distro_version": "f37", "force": False, "limit": "8"}
        )

        captured = capsys.readouterr()
        words = ("f36", "f38")
        assert all(word not in captured.out for word in words)

    def test_invalid_distro(self, capsys):
        search_term = "kernel"

        with raises(SystemExit):
            gpkg.main(
                {
                    "name": search_term,
                    "distro_version": "d11",
                    "force": True,
                    "limit": "2",
                }
            )

        captured = capsys.readouterr()
        assert captured.out == "Error: Invalid Distribution. Format: f{version}\n"

    def search_term_does_not_exist(self, capsys):
        search_term = "linux_in_toaster"

        gpkg.main(
            {"name": search_term, "distro_version": "f", "force": False, "limit": "8"}
        )

        captured = capsys.readouterr()
        assert captured.out == "No Updates Found. Check your arguments."

    def test_invalid_limit(self, capsys):
        search_term = "openjdk"

        with raises(SystemExit):
            gpkg.main(
                {
                    "name": search_term,
                    "distro_version": "f",
                    "force": False,
                    "limit": "easy",
                }
            )

        captured = capsys.readouterr()
        assert captured.out == "You must enter an integer value.\n"

    def test_limit_too_high(self, capsys):
        search_term = "openjdk"

        gpkg.main(
            {"name": search_term, "distro_version": "f", "force": False, "limit": "8"}
        )

        captured = capsys.readouterr()
        words = (search_term, "Update ID", "Package Name", "Status")
        not_words = ("No Updates", "Check your arguments")
        assert all(word in captured.out for word in words) and all(
            not_word not in captured.out for not_word in not_words
        )



import os

from pathlib import Path
from pytest import raises
from tempfile import gettempdir

from gpkgstatus.URLReader import URLReader


"""
These tests require an working internet connection.
"""


class TestURLReader:
    def test_site_that_exists(self):
        assert isinstance(URLReader("https://www.python.org"), URLReader)

    def test_site_404_not_found(self):
        with raises(Exception):
            URLReader("https://xkcd.com/404")

    def test_site_does_not_exist(self):
        with raises(SystemExit):
            URLReader("https://websitenotexists.py/")


class TestSavedFiles:
    def test_saved_file_exists(self):
        filename = "test_file.json"
        # python.org
        URLReader("https://2p66nmmycsj3.statuspage.io/api/v2/status.json").save_as_file(
            filename
        )

        path = Path(os.path.join(gettempdir(), filename))
        exists = path.exists()
        if exists:
            os.remove(path)
        assert exists

    def test_saved_is_file(self):
        filename = "test_file.json"
        # console status on python.org
        URLReader(
            "https://console.python.org/python-dot-org-live-consoles-status"
        ).save_as_file(filename)

        path = Path(os.path.join(gettempdir(), filename))
        is_file = path.is_file()
        if is_file:
            os.remove(path)
        assert is_file
